/*
 * Copyright (C) 1993, 1994 Marc Parmet.
 * This file is part of the Macintosh port of GNU Emacs.
 *
 * GNU Emacs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <setjmp.h>
#include <stdarg.h>
#include "stdio.h"
#include "signal.h"

#if defined(THINK_C)
#pragma options(check_ptrs)
#endif

#define MAXDESC 32
#define MAXPROC 10

#define LMStkLowPt		(*(Ptr *)0x110)
#define LMHiHeapMark	(*(Ptr *)0xBAE)

#if defined(powerc)
enum {
	unix_entry_ProcInfo = kCStackBased
	 | RESULT_SIZE(SIZE_CODE(sizeof(int)))
	 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(int)))
	 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(int *)))
	 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(va_list))),
};
#endif

extern int current_process_index;

/* These are indices into the jmp_buf type.  We use these to
   read and modify the state of a suspended thread.  This is
   non-portable. */
#if defined(THINK_C)
enum {
	jmp_buf_pc = 5,
	jmp_buf_sp = 10,
	jmp_buf_a4 = 8
};
#elif defined(powerc)
enum {
	jmp_buf_pc = 0,
	jmp_buf_sp = 2,
	jmp_buf_toc = 3
};
#else
/* For now, we leave these constants undefined, and any version compiled
   under a strange compiler will have to leave out the pieces of code
   that depend on the constants. */
#endif

#if defined(powerc)
struct linkage_area {
	long rtoc,reserved1,reserved2,lr,cr,sp;
};
#endif

extern struct proc {
	int pid;
	int ppid;
	char zombie;
	char vforked;
	int status;
	short homeResFile;
	char **heapHandle;
	char *stackbottom;
	int alarm_time;
	int start_time;
	char *cwd_string;
	int *errno_address;
	char ***environ_address;
	FILE iob[_NFILE];
	struct savearea {
		struct savearea_regs {
			jmp_buf jbuf;
			int d0_or_r3;
			int (*sigfunc)();
			int from_vfork;
		} regs;
		struct savearea_lomem {
			GrafPtr port;
			short curResFile;
			THz zone;
			Ptr ApplLimit,HeapEnd,HiHeapMark;
		} lomem;
	} savearea;
	struct fd {
		int flavor;
		long (*dispatch)(long,long,long,char *,long,struct savearea_lomem *);
		struct unix_object **object;
	} fd[MAXDESC];
	int (*sig[NSIG+1])();
} proctable[MAXPROC];

extern struct unix_object {
	struct unix_object **next,**prev;
	int flavor;
	int count;
	char translate;
	union {
		struct {
			int pipeNum;
			int rp,wp;
			char *pipeBuf;
		} pipe;
		struct {
			FSSpec spec;
			short refNum;
		} file;
		struct {
			WindowPtr w;
			TEHandle teh;
		} tty;
	} u;
} **object_list;

struct unix_object **lookup_object(long,long);
struct unix_object **add_object(long,long);
long add_fd(long,long (*)(long,long,long,char *,long,struct savearea_lomem *),
	struct unix_object **);
long close_internal(long,long,struct savearea_lomem *);
long read_write_internal(long,long,char *,long,struct savearea_lomem *);
long fflush_internal(FILE *,struct savearea_lomem *);
void fclose_internal(FILE *,struct savearea_lomem *);
void exit_internal(long);
long close_all_internal(long);
void _exit_internal(long);
void run_another_process(void);
long sleep_internal(long);
long wait3_internal(long *,long,long);
long pause_internal(void);

enum {
	fd_flavor_unused,
	fd_flavor_dev_null,
	fd_flavor_dev_tty,
	fd_flavor_file,
	fd_flavor_pipe
};

enum {
	object_flavor_file,
	object_flavor_dev_null,
	object_flavor_dev_tty,
	object_flavor_pipe
};
