;;;
;;; This file is part of a Macintosh port of GNU Emacs.
;;; Copyright (C) 1993, 1994 Marc Parmet.  All rights reserved.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;

;;;
;;; This file is read after .emacs is read at startup.
;;;

(require 'mac-runtime "mac/runtime")

(defvar mac-raw-map (make-keymap) "*Keymap for C-x * encoded keyboard")

; The keyboard driver looks at the virtual key code for every keystroke.
; If the code is less than 100 octal, then the ASCII code for that key
; is passed up to Emacs.  If the code is greater than or equal to
; 100 octal, the sequence
;   C-x * virtual-code
; is passed up, with 100 octal subtracted from the virtual code.  If the
; shift key is held down, that 100 is not subtracted.
;
; This encoding will change as the needs and problems of keymapping on
; the Macintosh become better understood.
; 
; Codes below are for ADB keyboards unless otherwise noted.

(define-key mac-raw-map "\000" nil) ; no such key
(define-key mac-raw-map "\001" ".") ; keypad .
(define-key mac-raw-map "\002" 'forward-char) ; non-ADB right cursor
(define-key mac-raw-map "\003" "*") ; keypad *
(define-key mac-raw-map "\004" nil) ; no such key
(define-key mac-raw-map "\005" "+") ; keypad +
(define-key mac-raw-map "\006" 'backward-char) ; non-ADB left cursor
(define-key mac-raw-map "\007" 'apple-clear) ; clear
(define-key mac-raw-map "\010" 'next-line) ; non-ADB down cursor
(define-key mac-raw-map "\011" nil) ; no such key
(define-key mac-raw-map "\012" nil) ; no such key
(define-key mac-raw-map "\013" "/") ; keypad /
(define-key mac-raw-map "\014" "\n") ; enter
(define-key mac-raw-map "\015" 'previous-line) ; non-ADB up cursor
(define-key mac-raw-map "\016" "-") ; keypad -
(define-key mac-raw-map "\017" nil) ; no such key
(define-key mac-raw-map "\020" nil) ; no such key
(define-key mac-raw-map "\021" "=") ; keypad =
(define-key mac-raw-map "\022" "0") ; keypad 0
(define-key mac-raw-map "\023" "1") ; keypad 1
(define-key mac-raw-map "\024" "2") ; keypad 2
(define-key mac-raw-map "\025" "3") ; keypad 3
(define-key mac-raw-map "\026" "4") ; keypad 4
(define-key mac-raw-map "\027" "5") ; keypad 5
(define-key mac-raw-map "\030" "6") ; keypad 6
(define-key mac-raw-map "\031" "7") ; keypad 7
(define-key mac-raw-map "\032" 'nil) ; no such key
(define-key mac-raw-map "\033" "8") ; keypad 8
(define-key mac-raw-map "\034" "9") ; keypad 9
(define-key mac-raw-map "\035" 'nil) ; no such key
(define-key mac-raw-map "\036" 'nil) ; no such key
(define-key mac-raw-map "\037" 'nil) ; no such key
(define-key mac-raw-map "\040" 'nil) ; F5
(define-key mac-raw-map "\041" 'nil) ; F6
(define-key mac-raw-map "\042" 'nil) ; F7
(define-key mac-raw-map "\043" 'apple-copy) ; F3, copy
(define-key mac-raw-map "\044" 'nil) ; F8
(define-key mac-raw-map "\045" 'nil) ; F9
(define-key mac-raw-map "\046" 'nil) ; F13
(define-key mac-raw-map "\047" 'nil) ; F11
(define-key mac-raw-map "\050" 'nil) ; no such key
(define-key mac-raw-map "\051" 'nil) ; F13
(define-key mac-raw-map "\052" 'nil) ; no such key
(define-key mac-raw-map "\053" 'nil) ; F14
(define-key mac-raw-map "\054" 'nil) ; no such key
(define-key mac-raw-map "\055" 'nil) ; F10
(define-key mac-raw-map "\056" 'nil) ; no such key
(define-key mac-raw-map "\057" 'nil) ; no such key
(define-key mac-raw-map "\060" 'nil) ; no such key
(define-key mac-raw-map "\061" 'nil) ; F15
(define-key mac-raw-map "\062" 'help-for-help) ; help
(define-key mac-raw-map "\063" 'beginning-of-buffer) ; home
(define-key mac-raw-map "\064" 'scroll-down) ; page up
(define-key mac-raw-map "\065" 'delete-char) ; del
(define-key mac-raw-map "\066" 'apple-paste) ; F4, paste
(define-key mac-raw-map "\067" 'end-of-buffer) ; end
(define-key mac-raw-map "\070" 'apple-cut) ; F2, cut
(define-key mac-raw-map "\071" 'scroll-up) ; page down
(define-key mac-raw-map "\072" 'apple-undo) ; F1, undo
(define-key mac-raw-map "\073" 'backward-char) ; left cursor
(define-key mac-raw-map "\074" 'forward-char) ; right cursor
(define-key mac-raw-map "\075" 'next-line) ; down cursor
(define-key mac-raw-map "\076" 'previous-line) ; up cursor
(define-key mac-raw-map "\077" 'nil) ; power switch
(define-key mac-raw-map "\102" "*") ; non-ADB keypad *
(define-key mac-raw-map "\106" "+") ; non-ADB keypad +
(define-key mac-raw-map "\110" "=") ; non-ADB keypad =
(define-key mac-raw-map "\115" "/") ; non-ADB keypad /

(define-key ctl-x-map "*" mac-raw-map)

;;; Since .emacs gets loaded before this file, a hook is supplied
;;; for you to put your own bindings in.

(defvar mac-raw-map-hooks nil
  "List of forms to evaluate after setting mac-raw-map.
This list is processed by: (mapcar 'eval mac-raw-map-hooks)")

(mapcar 'eval mac-raw-map-hooks)
